reorganize some code, add an extra command for adding unnamed sections
[project/uci.git] / uci.h
diff --git a/uci.h b/uci.h
index 43270c4..8f4fa71 100644 (file)
--- a/uci.h
+++ b/uci.h
@@ -157,6 +157,15 @@ extern int uci_cleanup(struct uci_context *ctx);
 extern int uci_lookup(struct uci_context *ctx, struct uci_element **res, struct uci_package *package, char *section, char *option);
 
 /**
+ * uci_add_section: Add an unnamed section
+ * @ctx: uci context
+ * @p: package to add the section to
+ * @type: section type
+ * @res: pointer to store a reference to the new section in
+ */
+extern int uci_add_section(struct uci_context *ctx, struct uci_package *p, char *type, struct uci_section **res);
+
+/**
  * uci_set_element_value: Replace an element's value with a new one
  * @ctx: uci context
  * @element: pointer to an uci_element struct pointer
@@ -174,8 +183,9 @@ extern int uci_set_element_value(struct uci_context *ctx, struct uci_element **e
  * @section: section name
  * @option: option name
  * @value: value (option) or type (section)
+ * @result: store the updated element in this variable (optional)
  */
-extern int uci_set(struct uci_context *ctx, struct uci_package *p, char *section, char *option, char *value);
+extern int uci_set(struct uci_context *ctx, struct uci_package *p, char *section, char *option, char *value, struct uci_element **result);
 
 /**
  * uci_rename: Rename an element
@@ -251,6 +261,24 @@ extern int uci_set_confdir(struct uci_context *ctx, const char *dir);
  */
 extern int uci_add_history_path(struct uci_context *ctx, const char *dir);
 
+/**
+ * uci_revert: revert all changes to a config item
+ * @ctx: uci context
+ * @p: pointer to a uci_package struct ptr (will be replaced by the revert)
+ * @section: section name (optional)
+ * @option option name (optional)
+ */
+extern int uci_revert(struct uci_context *ctx, struct uci_package **p, char *section, char *option);
+
+/**
+ * uci_parse_argument: parse a shell-style argument, with an arbitrary quoting style
+ * @ctx: uci context
+ * @stream: input stream
+ * @str: pointer to the current line (use NULL for parsing the next line)
+ * @result: pointer for the result
+ */
+extern int uci_parse_argument(struct uci_context *ctx, FILE *stream, char **str, char **result);
+
 /* UCI data structures */
 enum uci_type {
        UCI_TYPE_HISTORY = 0,
@@ -261,9 +289,10 @@ enum uci_type {
 };
 
 enum uci_flags {
-       UCI_FLAG_STRICT =      (1 << 0), /* strict mode for the parser */
-       UCI_FLAG_PERROR =      (1 << 1), /* print parser error messages */
-       UCI_FLAG_EXPORT_NAME = (1 << 2), /* when exporting, name unnamed sections */
+       UCI_FLAG_STRICT =        (1 << 0), /* strict mode for the parser */
+       UCI_FLAG_PERROR =        (1 << 1), /* print parser error messages */
+       UCI_FLAG_EXPORT_NAME =   (1 << 2), /* when exporting, name unnamed sections */
+       UCI_FLAG_SAVED_HISTORY = (1 << 3), /* store the saved history in memory as well */
 };
 
 struct uci_element
@@ -299,23 +328,6 @@ struct uci_context
        int bufsz;
 };
 
-struct uci_parse_context
-{
-       /* error context */
-       const char *reason;
-       int line;
-       int byte;
-
-       /* private: */
-       struct uci_package *package;
-       struct uci_section *section;
-       bool merge;
-       FILE *file;
-       const char *name;
-       char *buf;
-       int bufsz;
-};
-
 struct uci_package
 {
        struct uci_element e;
@@ -327,6 +339,7 @@ struct uci_package
        /* private: */
        int n_section;
        struct uci_list history;
+       struct uci_list saved_history;
 };
 
 struct uci_section